/** @file
 * @defgroup BNO055.c
 * @{
 *
 */

#include <BNO055.h>

// BNO055 Register Map
//
// BNO055 Page 0
#define BNO055_QUA_DATA_X_LSB   0x22
#define BNO055_LIA_DATA_Y_LSB   0x2A
#define BNO055_EUL_HEADING_LSB  0x1A	// yaw - pitch - roll

#define BNO055_OPERATION_MODE_NDOF	0x0C

#define MAX_TEST_INIT_BNO055		5

#define QUATERNION_2_POWER_14		16384

/* VARIABLE */
bno055_data 	data_BNO055 ;

/* EXTERN */
extern const	nrf_drv_twi_t m_twi ;
extern bool		update_data_BNO055 ;

/**************************************************************************/
/*!
    Inizilizzo il sensore
*/
/**************************************************************************/
bool	BNO055_init ( void )
{	
	uint8_t 	index_init_BNO055 ;
	
	// Pausa di attesa
	nrf_delay_ms ( 1000 ) ;
	
	/******************** Make sure we have the right device. *******************
	*/		
	uint8_t 	id ;
			
	uint8_t 	buff_id [ MAX_BUFF_SEND_BNO055 ] = "" ;
	
	buff_id [ 0 ] = BNO055_CHIP_ID_ADDR ;
	
	nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , buff_id , sizeof ( uint8_t ) , false ) ;		
	
	nrf_drv_twi_rx ( &m_twi , ADDR_BNO055 , &id , sizeof ( id ) ) ;		
	
	if( id != BNO055_ID )
	{		
		for ( index_init_BNO055 = 0 ; index_init_BNO055 < MAX_TEST_INIT_BNO055 ; index_init_BNO055 ++ )
		{		
			nrf_delay_ms ( 500 ) ; // hold on for boot
			
			nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , buff_id , 1 , false ) ;	
			
			nrf_drv_twi_rx ( &m_twi , ADDR_BNO055 , &id , 1 ) ;
			
			// ID corretto
			if ( id == BNO055_ID ) 
				break	; 			
			
		}
		
		// ID non corretto
		if ( id != BNO055_ID ) 
			return false ;  
		
	}	
	
	/** Switch to config mode (just in case since this is the default). *********
	*/
	
	// Dopo il power up il dispositivo entra automaticamente in CONFIG MODE
	
//		uint8_t 	*buff_mode ;
//		
//		buff_mode [ 0 ] = BNO055_OPR_MODE_ADDR ;
//		buff_mode [ 1 ] = OPERATION_MODE_CONFIG ;

//		nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , buff_mode , 2 , false ) ;
//		
//		nrf_drv_twi_rx ( &m_twi , ADDR_BNO055 , &mode_in , 1 ) ;

	/*********************************** Reset. *********************************
	*/
	
	uint8_t 	* buff_reset ;
	uint8_t 	counter_read_id_init ;
	
	// Resetto la var id
	id = 0x00 ;
	
	buff_reset [ 0 ] = BNO055_OPR_MODE_ADDR ;
	buff_reset [ 1 ] = 0x20 ;

	nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , buff_reset , 2 , false ) ;	

	// Non leggo la risposta ma inserisco una pausa
	
//	while( ( id != BNO055_ID ) && ( counter_read_id_init < 10 ) )
//	{
//		nrf_drv_twi_rx ( &m_twi , ADDR_BNO055 , &id , sizeof ( id ) ) ;
//		
//		nrf_delay_ms ( 10 ) ;
//		
//		counter_read_id_init ++ ;
//		
//	}
//	
//	if ( counter_read_id_init >= 10 )
//		return false ;
	
	nrf_delay_ms ( 50 ) ;		
	
	/****************************************************************************
	*/
	
	uint8_t 	buff_trigger [ MAX_BUFF_SEND_BNO055 ] = "" ;
	
	buff_trigger [ 0 ] = BNO055_SYS_TRIGGER_ADDR ;
	buff_trigger [ 1 ] = 0x00 ;

	nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , buff_trigger , 2 , false ) ;

	nrf_delay_ms ( 10 ) ;			
	
	/******************** Set to normal power mode . ****************************
	*/
	
	uint8_t 	buff_pwr_mode [ MAX_BUFF_SEND_BNO055 ] = "" ;
	
	buff_pwr_mode [ 0 ] = BNO055_PWR_MODE_ADDR ;		
	buff_pwr_mode [ 1 ] = POWER_MODE_NORMAL ;
	
	nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , buff_pwr_mode , 2 , false ) ;

	nrf_delay_ms ( 10 ) ;	

	/******************** Calibration status read . *****************************
	*/
	
//	uint8_t 	calib ;
//	
//	uint8_t 	buff_calib [ MAX_BUFF_SEND_BNO055 ] = "" ;
//	
//	buff_calib [ 0 ] = BNO055_CALIB_STAT ;		
//	
//	nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , buff_calib , 1 , false ) ;	
//	
//	nrf_drv_twi_rx ( &m_twi , ADDR_BNO055 , &calib , 1 ) ;
	
	// Calibrazione KO
//	if ( calib == 0x00 )	
//	{
//		for ( index_init_BNO055 = 0 ; index_init_BNO055 < 5 ; index_init_BNO055 ++ )
//		{		
//			nrf_delay_ms ( 500 ) ; // hold on for boot
//			
//			nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , buff_calib , 1 , false ) ;	
//			
//			nrf_drv_twi_rx ( &m_twi , ADDR_BNO055 , &calib , 1 ) ;
//			
//			// ID corretto
//			if ( calib != 0x00 ) 
//				break	; 			
//			
//		}
//		
//		// ID non corretto
////		if ( calib == 0x00 )
////			return false ;  
//	
//	}	
		
	// Fusion mode Yaw, Pitch e Roll xxxx1000 ( pag.22 BST_BNO055_DS000_12.pdf ) 
	if ( setModeBNO055 ( BNO055_OPERATION_MODE_NDOF ) == false )
		return false ;	
		
	
	return true ;
	
}

/**************************************************************************/
/*!
    Setto la modalit di utilizzo
*/
/**************************************************************************/
bool	setModeBNO055 ( uint8_t tmp_mode )
{
	/******** Set the requested operating mode (see section 3.3). ***************
	*/
	uint8_t 	buff_mode_end [ 2 ] = "" ;	

	nrf_delay_ms ( 50 ) ;
	
	buff_mode_end [ 0 ] = BNO055_OPR_MODE_ADDR ;
	buff_mode_end [ 1 ] = tmp_mode ;
	
	nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , buff_mode_end , 2 , false ) ;

	nrf_delay_ms ( 500 ) ;

	/********** Check mode set correcly . ***************************************
	*/		
	
	// Richiedo il mode settato
	uint8_t 	buff_mode_req = BNO055_OPR_MODE_ADDR ;
	
	uint8_t	mode_in = 0 ;

	nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , &buff_mode_req , 1 , false ) ;
	
	nrf_delay_ms ( 20 ) ;	
	
	nrf_drv_twi_rx ( &m_twi , ADDR_BNO055 , &mode_in , 1 ) ;
	
	if ( tmp_mode != mode_in )
		return false ;
	
	// Mode settato correttamente
	return true ;
	
}




/**************************************************************************/
/*!
    @brief  Reads the sensor and returns the data as a sensors_event_t
*/
/**************************************************************************/
void 	getDataBNO055 ( bno055_data *event )
{
	uint8_t 	buffer [ 6 ] = "" ;	
	int16_t 	x = 0 , y = 0 , z = 0 ;
	uint8_t 	empy_buff [ 6 ] ;
		
	memset ( empy_buff , 0 , sizeof ( empy_buff ) ) ;
	
	// Nota : in modalit fusion (  fusion mode - IMU ) la frequenza  fissa a 100 Hz ( 'Table 3-14' pag.31 'BST_BNO055_DS000_12.pdf' )
	
  /* Read vector data (6 bytes) */
  nrf_drv_twi_rx ( &m_twi , ADDR_BNO055 , buffer , 6 ) ;
	
	if ( memcmp ( buffer , empy_buff , 6 ) != 0 )
	{
		memcpy ( &x , buffer , sizeof ( int16_t ) ) ; 
		memcpy ( &z , buffer + 2 , sizeof ( int16_t ) ) ; 
		memcpy ( &y , buffer + 4 , sizeof ( int16_t ) ) ; 
		
		/* Clear the event */
		memset ( event , 0 , sizeof ( bno055_data ) ) ;
		
		// Lo converto in gradi e inserisco sull'oggettos
		event->yaw = ( float ) x / 16. ;  
		event->roll = ( float ) y / 16. ;  
		event->pitch = ( float ) z / 16. ; 	
		
		// Dati aggiornati li invio all'app
		update_data_BNO055 = true ;
		
	}
	
}



/**************************************************************************/
/*!
    @brief  Reads from the chip BNO055 the Linear Acceleration Y
*/
/**************************************************************************/
void readLIADataYBNO055 ( bno055_data *event )
{
	uint8_t 	buffer [ 2 ] = "" ;	
	int16_t 	y = 0 ;
	uint8_t 	empy_buff [ 2 ] ;
	uint8_t 	req_lia_data_y = BNO055_LIA_DATA_Y_LSB ;
	
	memset ( empy_buff , 0 , sizeof ( empy_buff ) ) ;
	
	// Nota : in modalit fusion (  fusion mode - IMU ) la frequenza  fissa a 100 Hz ( 'Table 3-14' pag.31 'BST_BNO055_DS000_12.pdf' )
	
	// Richiedo al chip BNO055 l'accelerazione lineare Y
	nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , &req_lia_data_y , 1 , false ) ;
		
  /* Read vector data (2 bytes) */
  nrf_drv_twi_rx ( &m_twi , ADDR_BNO055 , buffer , 2 ) ;
	
	if ( memcmp ( buffer , empy_buff , 2 ) != 0 )
	{
		memcpy ( &y , buffer , sizeof ( int16_t ) ) ; 
		
		/* Clear the event */
		event->linear_acceleration_y = 0 ; 

		// Unit di misura m/s2
		event->linear_acceleration_y = ( float ) y ;  
		
		// Dati aggiornati li invio all'app
		update_data_BNO055 = true ;	
		
	}
	
}





/**************************************************************************/
/*!
    @brief  Reads from the chip BNO055 the Quaternion X
*/
/**************************************************************************/
void readQUADataXBNO055 ( bno055_data *event )
{
	uint8_t 	buffer [ 2 ] = "" ;	
	int16_t 	x = 0 ;
	uint8_t 	empy_buff [ 2 ] ;
	uint8_t 	req_qua_data_x = BNO055_QUA_DATA_X_LSB ;
	
	memset ( empy_buff , 0 , sizeof ( empy_buff ) ) ;
	
	// Nota : in modalit fusion (  fusion mode - IMU ) la frequenza  fissa a 100 Hz ( 'Table 3-14' pag.31 'BST_BNO055_DS000_12.pdf' )
	
	// Richiedo al chip BNO055 l'orientamento X
	nrf_drv_twi_tx ( &m_twi , ADDR_BNO055 , &req_qua_data_x , 1 , false ) ;
		
  /* Read vector data (2 bytes) */
  nrf_drv_twi_rx ( &m_twi , ADDR_BNO055 , buffer , 2 ) ;
	
	if ( memcmp ( buffer , empy_buff , 2 ) != 0 )
	{
		memcpy ( &x , buffer , sizeof ( int16_t ) ) ; 
		
		/* Clear the event */
		event->quaternion_x = 0 ;
		
		// Quaternion (unit less) 1 Quaternion (unit less) = 2^14 LSB ( pag.35 'BST_BNO055_DS000_12.pdf' )
		event->quaternion_x = ( float ) x / QUATERNION_2_POWER_14 ;  
		
		// Dati aggiornati li invio all'app
		update_data_BNO055 = true ;	
		
	}
	
}


// 


/** @} */
